"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnonCredsRsHolderService = void 0;
const anoncreds_1 = require("@aries-framework/anoncreds");
const core_1 = require("@aries-framework/core");
const anoncreds_shared_1 = require("@hyperledger/anoncreds-shared");
const AnonCredsRsError_1 = require("../errors/AnonCredsRsError");
let AnonCredsRsHolderService = class AnonCredsRsHolderService {
    async createLinkSecret(agentContext, options) {
        var _a;
        let masterSecret;
        try {
            masterSecret = anoncreds_shared_1.MasterSecret.create();
            // FIXME: This is a very specific format of anoncreds-rs. I think it should be simply a string
            const linkSecretJson = masterSecret.toJson();
            return {
                linkSecretId: (_a = options === null || options === void 0 ? void 0 : options.linkSecretId) !== null && _a !== void 0 ? _a : core_1.utils.uuid(),
                linkSecretValue: linkSecretJson.value.ms,
            };
        }
        finally {
            masterSecret === null || masterSecret === void 0 ? void 0 : masterSecret.handle.clear();
        }
    }
    async createProof(agentContext, options) {
        const { credentialDefinitions, proofRequest, selectedCredentials, schemas } = options;
        let presentation;
        try {
            const rsCredentialDefinitions = {};
            for (const credDefId in credentialDefinitions) {
                rsCredentialDefinitions[credDefId] = credentialDefinitions[credDefId];
            }
            const rsSchemas = {};
            for (const schemaId in schemas) {
                rsSchemas[schemaId] = schemas[schemaId];
            }
            const credentialRepository = agentContext.dependencyManager.resolve(anoncreds_1.AnonCredsCredentialRepository);
            // Cache retrieved credentials in order to minimize storage calls
            const retrievedCredentials = new Map();
            const credentialEntryFromAttribute = async (attribute) => {
                let credentialRecord = retrievedCredentials.get(attribute.credentialId);
                if (!credentialRecord) {
                    credentialRecord = await credentialRepository.getByCredentialId(agentContext, attribute.credentialId);
                    retrievedCredentials.set(attribute.credentialId, credentialRecord);
                }
                const revocationRegistryDefinitionId = credentialRecord.credential.rev_reg_id;
                const revocationRegistryIndex = credentialRecord.credentialRevocationId;
                // TODO: Check if credential has a revocation registry id (check response from anoncreds-rs API, as it is
                // sending back a mandatory string in Credential.revocationRegistryId)
                const timestamp = attribute.timestamp;
                let revocationState;
                let revocationRegistryDefinition;
                try {
                    if (timestamp && revocationRegistryIndex && revocationRegistryDefinitionId) {
                        if (!options.revocationRegistries[revocationRegistryDefinitionId]) {
                            throw new AnonCredsRsError_1.AnonCredsRsError(`Revocation Registry ${revocationRegistryDefinitionId} not found`);
                        }
                        const { definition, tailsFilePath } = options.revocationRegistries[revocationRegistryDefinitionId];
                        revocationRegistryDefinition = anoncreds_shared_1.RevocationRegistryDefinition.fromJson(definition);
                        revocationState = anoncreds_shared_1.CredentialRevocationState.create({
                            revocationRegistryIndex: Number(revocationRegistryIndex),
                            revocationRegistryDefinition,
                            tailsPath: tailsFilePath,
                            revocationStatusList: anoncreds_shared_1.RevocationStatusList.create({
                                issuerId: definition.issuerId,
                                issuanceByDefault: true,
                                revocationRegistryDefinition,
                                revocationRegistryDefinitionId,
                                timestamp,
                            }),
                        });
                    }
                    return {
                        linkSecretId: credentialRecord.linkSecretId,
                        credentialEntry: {
                            credential: credentialRecord.credential,
                            revocationState: revocationState === null || revocationState === void 0 ? void 0 : revocationState.toJson(),
                            timestamp,
                        },
                    };
                }
                finally {
                    revocationState === null || revocationState === void 0 ? void 0 : revocationState.handle.clear();
                    revocationRegistryDefinition === null || revocationRegistryDefinition === void 0 ? void 0 : revocationRegistryDefinition.handle.clear();
                }
            };
            const credentialsProve = [];
            const credentials = [];
            let entryIndex = 0;
            for (const referent in selectedCredentials.attributes) {
                const attribute = selectedCredentials.attributes[referent];
                credentials.push(await credentialEntryFromAttribute(attribute));
                credentialsProve.push({ entryIndex, isPredicate: false, referent, reveal: attribute.revealed });
                entryIndex = entryIndex + 1;
            }
            for (const referent in selectedCredentials.predicates) {
                const predicate = selectedCredentials.predicates[referent];
                credentials.push(await credentialEntryFromAttribute(predicate));
                credentialsProve.push({ entryIndex, isPredicate: true, referent, reveal: true });
                entryIndex = entryIndex + 1;
            }
            // Get all requested credentials and take linkSecret. If it's not the same for every credential, throw error
            const linkSecretsMatch = credentials.every((item) => item.linkSecretId === credentials[0].linkSecretId);
            if (!linkSecretsMatch) {
                throw new AnonCredsRsError_1.AnonCredsRsError('All credentials in a Proof should have been issued using the same Link Secret');
            }
            const linkSecretRecord = await agentContext.dependencyManager
                .resolve(anoncreds_1.AnonCredsLinkSecretRepository)
                .getByLinkSecretId(agentContext, credentials[0].linkSecretId);
            if (!linkSecretRecord.value) {
                throw new AnonCredsRsError_1.AnonCredsRsError('Link Secret value not stored');
            }
            presentation = anoncreds_shared_1.Presentation.create({
                credentialDefinitions: rsCredentialDefinitions,
                schemas: rsSchemas,
                presentationRequest: proofRequest,
                credentials: credentials.map((entry) => entry.credentialEntry),
                credentialsProve,
                selfAttest: selectedCredentials.selfAttestedAttributes,
                masterSecret: { value: { ms: linkSecretRecord.value } },
            });
            return presentation.toJson();
        }
        finally {
            presentation === null || presentation === void 0 ? void 0 : presentation.handle.clear();
        }
    }
    async createCredentialRequest(agentContext, options) {
        const { credentialDefinition, credentialOffer } = options;
        let createReturnObj;
        try {
            const linkSecretRepository = agentContext.dependencyManager.resolve(anoncreds_1.AnonCredsLinkSecretRepository);
            // If a link secret is specified, use it. Otherwise, attempt to use default link secret
            const linkSecretRecord = options.linkSecretId
                ? await linkSecretRepository.getByLinkSecretId(agentContext, options.linkSecretId)
                : await linkSecretRepository.findDefault(agentContext);
            if (!linkSecretRecord) {
                // No default link secret
                throw new AnonCredsRsError_1.AnonCredsRsError('No link secret provided to createCredentialRequest and no default link secret has been found');
            }
            createReturnObj = anoncreds_shared_1.CredentialRequest.create({
                entropy: anoncreds_shared_1.anoncreds.generateNonce(),
                credentialDefinition: credentialDefinition,
                credentialOffer: credentialOffer,
                masterSecret: { value: { ms: linkSecretRecord.value } },
                masterSecretId: linkSecretRecord.linkSecretId,
            });
            return {
                credentialRequest: createReturnObj.credentialRequest.toJson(),
                credentialRequestMetadata: createReturnObj.credentialRequestMetadata.toJson(),
            };
        }
        finally {
            createReturnObj === null || createReturnObj === void 0 ? void 0 : createReturnObj.credentialRequest.handle.clear();
            createReturnObj === null || createReturnObj === void 0 ? void 0 : createReturnObj.credentialRequestMetadata.handle.clear();
        }
    }
    async storeCredential(agentContext, options) {
        var _a, _b;
        const { credential, credentialDefinition, credentialRequestMetadata, revocationRegistry, schema } = options;
        const linkSecretRecord = await agentContext.dependencyManager
            .resolve(anoncreds_1.AnonCredsLinkSecretRepository)
            .getByLinkSecretId(agentContext, credentialRequestMetadata.master_secret_name);
        const revocationRegistryDefinition = revocationRegistry === null || revocationRegistry === void 0 ? void 0 : revocationRegistry.definition;
        const credentialId = (_a = options.credentialId) !== null && _a !== void 0 ? _a : core_1.utils.uuid();
        let credentialObj;
        let processedCredential;
        try {
            credentialObj = anoncreds_shared_1.Credential.fromJson(credential);
            processedCredential = credentialObj.process({
                credentialDefinition: credentialDefinition,
                credentialRequestMetadata: credentialRequestMetadata,
                masterSecret: { value: { ms: linkSecretRecord.value } },
                revocationRegistryDefinition,
            });
            const credentialRepository = agentContext.dependencyManager.resolve(anoncreds_1.AnonCredsCredentialRepository);
            await credentialRepository.save(agentContext, new anoncreds_1.AnonCredsCredentialRecord({
                credential: processedCredential.toJson(),
                credentialId,
                linkSecretId: linkSecretRecord.linkSecretId,
                issuerId: options.credentialDefinition.issuerId,
                schemaName: schema.name,
                schemaIssuerId: schema.issuerId,
                schemaVersion: schema.version,
                credentialRevocationId: (_b = processedCredential.revocationRegistryIndex) === null || _b === void 0 ? void 0 : _b.toString(),
            }));
            return credentialId;
        }
        finally {
            credentialObj === null || credentialObj === void 0 ? void 0 : credentialObj.handle.clear();
            processedCredential === null || processedCredential === void 0 ? void 0 : processedCredential.handle.clear();
        }
    }
    async getCredential(agentContext, options) {
        const credentialRecord = await agentContext.dependencyManager
            .resolve(anoncreds_1.AnonCredsCredentialRepository)
            .getByCredentialId(agentContext, options.credentialId);
        const attributes = {};
        for (const attribute in credentialRecord.credential.values) {
            attributes[attribute] = credentialRecord.credential.values[attribute].raw;
        }
        return {
            attributes,
            credentialDefinitionId: credentialRecord.credential.cred_def_id,
            credentialId: credentialRecord.credentialId,
            schemaId: credentialRecord.credential.schema_id,
            credentialRevocationId: credentialRecord.credentialRevocationId,
            revocationRegistryId: credentialRecord.credential.rev_reg_id,
        };
    }
    async deleteCredential(agentContext, credentialId) {
        const credentialRepository = agentContext.dependencyManager.resolve(anoncreds_1.AnonCredsCredentialRepository);
        const credentialRecord = await credentialRepository.getByCredentialId(agentContext, credentialId);
        await credentialRepository.delete(agentContext, credentialRecord);
    }
    async getCredentialsForProofRequest(agentContext, options) {
        var _a;
        const proofRequest = options.proofRequest;
        const referent = options.attributeReferent;
        const requestedAttribute = (_a = proofRequest.requested_attributes[referent]) !== null && _a !== void 0 ? _a : proofRequest.requested_predicates[referent];
        if (!requestedAttribute) {
            throw new AnonCredsRsError_1.AnonCredsRsError(`Referent not found in proof request`);
        }
        const attributes = requestedAttribute.name ? [requestedAttribute.name] : requestedAttribute.names;
        const restrictionQuery = requestedAttribute.restrictions
            ? this.queryFromRestrictions(requestedAttribute.restrictions)
            : undefined;
        const query = Object.assign(Object.assign({ attributes }, restrictionQuery), options.extraQuery);
        const credentials = await agentContext.dependencyManager
            .resolve(anoncreds_1.AnonCredsCredentialRepository)
            .findByQuery(agentContext, query);
        return credentials.map((credentialRecord) => {
            const attributes = {};
            for (const attribute in credentialRecord.credential.values) {
                attributes[attribute] = credentialRecord.credential.values[attribute].raw;
            }
            return {
                credentialInfo: {
                    attributes,
                    credentialDefinitionId: credentialRecord.credential.cred_def_id,
                    credentialId: credentialRecord.credentialId,
                    schemaId: credentialRecord.credential.schema_id,
                    credentialRevocationId: credentialRecord.credentialRevocationId,
                    revocationRegistryId: credentialRecord.credential.rev_reg_id,
                },
                interval: proofRequest.non_revoked,
            };
        });
    }
    queryFromRestrictions(restrictions) {
        var _a, _b;
        const query = [];
        for (const restriction of restrictions) {
            const queryElements = {};
            if (restriction.cred_def_id) {
                queryElements.credentialDefinitionId = restriction.cred_def_id;
            }
            if (restriction.issuer_id || restriction.issuer_did) {
                queryElements.issuerId = (_a = restriction.issuer_id) !== null && _a !== void 0 ? _a : restriction.issuer_did;
            }
            if (restriction.rev_reg_id) {
                queryElements.revocationRegistryId = restriction.rev_reg_id;
            }
            if (restriction.schema_id) {
                queryElements.schemaId = restriction.schema_id;
            }
            if (restriction.schema_issuer_id || restriction.schema_issuer_did) {
                queryElements.schemaIssuerId = (_b = restriction.schema_issuer_id) !== null && _b !== void 0 ? _b : restriction.schema_issuer_did;
            }
            if (restriction.schema_name) {
                queryElements.schemaName = restriction.schema_name;
            }
            if (restriction.schema_version) {
                queryElements.schemaVersion = restriction.schema_version;
            }
            query.push(queryElements);
        }
        return query.length === 1 ? query[0] : { $or: query };
    }
};
AnonCredsRsHolderService = __decorate([
    (0, core_1.injectable)()
], AnonCredsRsHolderService);
exports.AnonCredsRsHolderService = AnonCredsRsHolderService;
//# sourceMappingURL=AnonCredsRsHolderService.js.map